#include "statistic.hpp"
#include <array>
#include <imgui.h>

Statistic::Statistic(const std::string& name, uint32_t sample_count) : name(name)
{
    this->samples.resize(sample_count, 0.0f);
}

void Statistic::interface()
{
    std::string plot_name = this->name;

    for (uint32_t index = 0; index < plot_name.size(); index++)
    {
        if (index == 0)
        {
            plot_name[index] = std::toupper(plot_name[index]);
        }

        else if (plot_name[index - 1] == '_')
        {
            plot_name[index - 1] = ' ';
            plot_name[index] = std::toupper(plot_name[index]);
        }
    }

    ImGui::PlotLines(plot_name.c_str(), this->samples.data(), this->samples.size(), this->current_index, nullptr, 0.0f, this->get_max(), ImVec2(0, 64));

    ImVec2 cursor = ImGui::GetCursorPos();
    ImGui::SetWindowFontScale(0.7f);

    ImGui::SetCursorPosX(cursor.x + 5);
    ImGui::SetCursorPosY(cursor.y - 22.0f);
    ImGui::Text("Avg: %8.4f", this->get_average());
    ImGui::SetCursorPosX(cursor.x + 90);
    ImGui::SetCursorPosY(cursor.y - 22.0f);
    ImGui::Text("Min: %8.4f", this->get_min());
    ImGui::SetCursorPosX(cursor.x + 175);
    ImGui::SetCursorPosY(cursor.y - 22.0f);
    ImGui::Text("Max: %8.4f", this->get_max());

    ImGui::SetCursorPos(cursor);
    ImGui::SetWindowFontScale(1.0);
}

void Statistic::add_sample(float sample)
{
    this->samples[this->current_index] = sample;
    this->current_index = (this->current_index + 1) % this->samples.size();
    this->valid_index = std::max(this->valid_index, this->current_index);
}

float Statistic::get_average() const
{
    if (this->valid_index <= 0)
    {
        return 0.0f;
    }

    float sample_sum = 0.0f;

    for (uint32_t index = 0; index < this->valid_index; index++)
    {
        sample_sum += this->samples[index];
    }

    return sample_sum / (float)this->valid_index;
}

float Statistic::get_min() const
{
    if (this->valid_index <= 0)
    {
        return 0.0f;
    }

    float sample_min = this->samples[0];

    for (uint32_t index = 0; index < this->valid_index; index++)
    {
        sample_min = std::min(sample_min, this->samples[index]);
    }

    return sample_min;
}

float Statistic::get_max() const
{
    if (this->valid_index <= 0)
    {
        return 0.0f;
    }

    float sample_max = this->samples[0];

    for (uint32_t index = 0; index < this->valid_index; index++)
    {
        sample_max = std::max(sample_max, this->samples[index]);
    }

    return sample_max;
}

const std::string& Statistic::get_name() const
{
    return this->name;
}

const std::vector<float>& Statistic::get_samples() const
{
    return this->samples;
}

Statistic::Ptr make_statistic(const std::string& name, uint32_t sample_count)
{
    return std::make_shared<Statistic>(name, sample_count);
}